package it.bluescreen.roboat;

import android.content.Context;
import android.content.Intent;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.graphics.Canvas;
import android.graphics.Color;
import android.graphics.Matrix;
import android.graphics.Paint;
import android.graphics.PixelFormat;
import android.graphics.PorterDuff;
import android.graphics.Rect;
import android.graphics.RectF;
import android.media.MediaPlayer;
import android.os.Build;
import android.support.annotation.RequiresApi;
import android.util.AttributeSet;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.MotionEvent;
import android.view.SurfaceHolder;
import android.view.SurfaceView;
import android.view.animation.AccelerateDecelerateInterpolator;
import android.widget.FrameLayout;
import android.widget.RelativeLayout;

import java.util.ArrayList;
import java.util.List;


public class GameView extends RelativeLayout implements Runnable {

    private Roboat roboat;
    private List<Obstacle> obstacleList;
    private GoalObject goalObject;
    private FrameLayout wave;
    private EnergyBar energyBar;

    boolean isPlaying;
    private Thread gameThread = null;
    private Paint paint;
    private Canvas canvas;
    private SurfaceHolder surfaceHolder;

    private Matrix matrix;
    int screenX;
    int screenY;

    private boolean isGameOver;
    private boolean isGameCompleted;
    private MediaPlayer baseGameSound;
    private MediaPlayer waveSound;
    private MediaPlayer gameWinSound;
    private MediaPlayer gameOverSound;

    public GameView(Context context) {
        super(context);
    }

    public GameView(Context context, AttributeSet attrs) {
        super(context, attrs);
    }

    public GameView(Context context, AttributeSet attrs, int defStyleAttr) {
        super(context, attrs, defStyleAttr);
    }

    @RequiresApi(api = Build.VERSION_CODES.LOLLIPOP)
    public GameView(Context context, AttributeSet attrs, int defStyleAttr, int defStyleRes) {
        super(context, attrs, defStyleAttr, defStyleRes);
    }


    public void initialize(int screenX, int screenY){
        LayoutInflater.from(getContext()).inflate(R.layout.view_game, this, true);
        setBackgroundColor(Color.TRANSPARENT);
        roboat = new Roboat(getContext(), screenX, screenY);
        SurfaceView surfaceView = (SurfaceView)findViewById(R.id.surfaceView);
        surfaceView.setZOrderOnTop(true);
        surfaceHolder = surfaceView.getHolder();
        surfaceHolder.setFormat(PixelFormat.TRANSPARENT);

        paint = new Paint();
        matrix = new Matrix();
        this.screenX = screenX;
        this.screenY = screenY;
        isGameOver = false;
        initializeForLevel(1);

        energyBar = new EnergyBar(getContext(), Color.GREEN, 100);
        wave = new FrameLayout(getContext());
        baseGameSound = MediaPlayer.create(getContext(), R.raw.base);
        waveSound = MediaPlayer.create(getContext(), R.raw.ontouch);
        gameOverSound = MediaPlayer.create(getContext() , R.raw.gameover);
        gameWinSound = MediaPlayer.create(getContext() , R.raw.gamewin);
        baseGameSound.start();
        baseGameSound.setLooping(true);
    }

    private void initializeForLevel(int level) {
        obstacleList = new ArrayList<>();
        if(level == 1){
            goalObject = new GoalObject(getContext(),  screenX, screenY, R.drawable.suitcase, 10, 10);

            Obstacle obstacle1 = new Obstacle(getContext(), screenX, screenY, R.drawable.radioactive, 50, 1400);
            Obstacle obstacle10 = new Obstacle(getContext(), screenX, screenY, R.drawable.radioactive, 550, 452);
            Obstacle obstacle3 = new Obstacle(getContext(), screenX, screenY, R.drawable.radioactive, 300, 840);
            Obstacle obstacle5 = new Obstacle(getContext(), screenX, screenY, R.drawable.radioactive, 500, 1000);
            //Obstacle obstacle7 = new Obstacle(getContext(), screenX, screenY, R.drawable.radioactive, 290, 400);

            Obstacle obstacle2 = new Obstacle(getContext(), screenX, screenY, R.drawable.rocks, 730, 1400);
            Obstacle obstacle4 = new Obstacle(getContext(), screenX, screenY, R.drawable.rocks, 400, 700);
            Obstacle obstacle6 = new Obstacle(getContext(), screenX, screenY, R.drawable.rocks, 800, 1000);
            Obstacle obstacle8 = new Obstacle(getContext(), screenX, screenY, R.drawable.rocks, 10, 260);
            Obstacle obstacle9 = new Obstacle(getContext(), screenX, screenY, R.drawable.rocks, 30, 1150);
            Obstacle obstacle11 = new Obstacle(getContext(), screenX, screenY, R.drawable.rocks, 730, 353);
            Obstacle obstacle12 = new Obstacle(getContext(), screenX, screenY, R.drawable.rocks, 650, 100);

            Obstacle obstacle13 = new Obstacle(getContext(), screenX, screenY, R.drawable.radioactive, 500, 145);
            Obstacle obstacle14 = new Obstacle(getContext(), screenX, screenY, R.drawable.radioactive, 30, 350);
            Obstacle obstacle15 = new Obstacle(getContext(), screenX, screenY, R.drawable.rocks, 700, 20);
            Obstacle obstacle16 = new Obstacle(getContext(), screenX, screenY, R.drawable.radioactive, 740, 30);
            Obstacle obstacle17 = new Obstacle(getContext(), screenX, screenY, R.drawable.rocks, 520, 10);
            obstacleList.add(obstacle1);
            obstacleList.add(obstacle2);
            obstacleList.add(obstacle3);
            obstacleList.add(obstacle4);
            obstacleList.add(obstacle5);
            obstacleList.add(obstacle6);
            //obstacleList.add(obstacle7);
            obstacleList.add(obstacle8);
            obstacleList.add(obstacle9);
            obstacleList.add(obstacle10);
            obstacleList.add(obstacle11);
            obstacleList.add(obstacle12);
         //   obstacleList.add(obstacle13);
         //   obstacleList.add(obstacle14);
         //   obstacleList.add(obstacle15);
         //   obstacleList.add(obstacle16);
         //   obstacleList.add(obstacle17);
        }
    }

    @Override public void run() {
        while (isPlaying) {
            update();
            draw();
            control();
        }
    }

    private void update() {
        roboat.update();
        energyBar.update();

        if(Rect.intersects(roboat.getPolygonArea(), goalObject.getPolygonArea())){
            gameWin();
        }
        if(!isGameCompleted) {
            if(thereIsCollision() || energyBar.value < 2){
                gameOver();
            }

            /*
            if (Rect.intersects(roboat.getPolygonArea(), obstacle.getPolygonArea())) {
                gameOver();
            }
            if (energyBar.value < 2) {
                gameOver();
            }
            */
        }

/*
        if(collisione(roboat.getPolygonArea(), obstacle.getPolygonArea())){
            gameOver();
        }
*/
    }

    private boolean thereIsCollision() {
        for(Obstacle o : obstacleList){
            if(Rect.intersects(roboat.getPolygonArea(), o.getPolygonArea())){
                return true;
            }
        }
        return false;
    }

    private void gameWin() {
        isPlaying = false;
        isGameCompleted = true;
        baseGameSound.stop();
        gameWinSound.start();
    }

    private boolean collisione(Rect rect1, Rect rect2){

        Log.d("COLLIDE", "rect 1 = " + rect1.toString());
        Log.d("COLLIDE", "rect 2 = " + rect2.toString());


        return rect1.left < rect2.right &&
        rect1.right > rect2.left &&
        rect1.top < rect2.top &&
        rect1.bottom > rect2.top;
    }

    private void gameOver() {
        isPlaying = false;
        isGameOver = true;
        baseGameSound.stop();
        gameOverSound.start();
    }

    private void draw() {
        if (surfaceHolder.getSurface().isValid()) {
            canvas = surfaceHolder.lockCanvas();
            canvas.drawColor(Color.TRANSPARENT, PorterDuff.Mode.CLEAR);
            drawRobot();
           // canvas.drawRect(roboat.getPolygonArea(), paint);
/*
            canvas.drawBitmap(roboat.getBitmap(), null,
                    new RectF(roboat.getX(), roboat.getY(), roboat.getX() + Roboat.ROBOAT_WIDTH, roboat.getY() + Roboat.ROBOAT_HEIGHT),
                    null);
*/

            for(Obstacle obstacle : obstacleList) {
                canvas.drawBitmap(obstacle.getBitmap(), obstacle.getX(), obstacle.getY(), paint);
            }
            canvas.drawBitmap(goalObject.getBitmap(), goalObject.getX(), goalObject.getY(), paint);
          //  canvas.drawRect(obstacle.getPolygonArea(), paint);


            drawRobot();
            drawEnergyBar();
            if(isGameOver){
                Bitmap gameOver1 = BitmapFactory.decodeResource(getResources(), R.drawable.gameover);
                Bitmap gameOver2 = Bitmap.createScaledBitmap(gameOver1, 1000, 180, false);

                canvas.drawBitmap(gameOver2, 40, canvas.getHeight()/2, paint);
            }

            if(isGameCompleted){
                Bitmap gameOver1 = BitmapFactory.decodeResource(getResources(), R.drawable.finale);
                Bitmap gameOver2 = Bitmap.createScaledBitmap(gameOver1, 1080, 1920, false);

                canvas.drawBitmap(gameOver2, 0, 0, paint);
            }

            surfaceHolder.unlockCanvasAndPost(canvas);
        }
    }

    private void drawEnergyBar() {
        Paint mPaint = new Paint();
        canvas.drawBitmap(energyBar.bitmap, null,
                new RectF(45, 1780, 145, 1880),
                null);
        mPaint.setStyle(Paint.Style.STROKE);
        mPaint.setStrokeCap(Paint.Cap.ROUND);
        mPaint.setStrokeWidth(16);
        mPaint.setColor(energyBar.color);
        canvas.drawLine(energyBar.left, energyBar.bottom ,
                (float) (energyBar.left + (energyBar.value/100 * (energyBar.right - energyBar.left))), energyBar.bottom, mPaint);

    }

    private void drawRobot() {
        /*
        Paint paint = new Paint();
        paint.setStyle(Paint.Style.FILL);
        paint.setColor(Color.YELLOW);
        canvas.drawCircle(roboat.getX(), roboat.getYTranslated(), 20, paint);

        paint.setColor(Color.GREEN);
        canvas.drawCircle(roboat.getCodaX(), roboat.getCodaYTranslated(), 20, paint);

        float px = roboat.getCenterX();
        float py = roboat.getCenterY();
        paint.setColor(Color.RED);
        canvas.drawCircle(px, py, 20, paint);

        if(roboat.mustRotate()) {
            int rotation = 1;
            matrix.postTranslate(-px, -py);
            matrix.postRotate(rotation);
            matrix.postTranslate(px, py);
            roboat.changeOrientation(-1);
        }else{
            matrix.reset();
            matrix.postTranslate(-px, -py);
            matrix.postRotate((float) (90 - roboat.getOrientation()));
            matrix.postTranslate(px, py);
            matrix.postTranslate(roboat.getX() - Roboat.ROBOAT_WIDTH/2, roboat.getYTranslated());
        }

*/
        matrix.reset();
        matrix.postTranslate(roboat.getX() - Roboat.ROBOAT_WIDTH/2, roboat.getYTranslated());
        canvas.drawBitmap(roboat.getBitmap(), matrix, null);
    }

    private void control() {
        try {
            gameThread.sleep(14);
        } catch (InterruptedException e) {
            e.printStackTrace();
        }
    }

    public void pause() {
        isPlaying = false;
        try {
            gameThread.join();
        } catch (InterruptedException e) {
        }
    }

    public void resume() {
        isPlaying = true;
        gameThread = new Thread(this);
        gameThread.start();
        baseGameSound.start();
    }

    @Override public boolean onTouchEvent(MotionEvent motionEvent) {
        switch (motionEvent.getAction() & MotionEvent.ACTION_MASK) {
            case MotionEvent.ACTION_DOWN:
                roboat.setAccelerating(true);
                roboat.moveToXY(motionEvent.getX(), motionEvent.getY());
                energyBar.setIsConsuming(true);
                startWaveAnimation(motionEvent.getX(), motionEvent.getY());
                break;
            case MotionEvent.ACTION_UP:
                energyBar.setIsConsuming(false);
                roboat.setAccelerating(false);
                break;

        }
        if(isGameOver){
            if(motionEvent.getAction()==MotionEvent.ACTION_DOWN){
                System.exit(0);
            }
        }
        if(isGameCompleted){
            if(motionEvent.getAction()==MotionEvent.ACTION_DOWN){
                System.exit(0);            }
        }
        return true;
    }

    private void startWaveAnimation(float x, float y) {
        RelativeLayout.LayoutParams params = new RelativeLayout.LayoutParams(100,100);
        params.leftMargin = ((int)x);
        params.topMargin = ((int)y);
        removeView(wave);
        FrameLayout wave = new FrameLayout(getContext());
        wave.setBackgroundResource(R.drawable.wave);
        addView(wave, params);
        waveSound.start();
        wave.animate().alpha(0f).setInterpolator(new AccelerateDecelerateInterpolator()).setDuration(2000);
        wave.animate().scaleX(100f).setInterpolator(new AccelerateDecelerateInterpolator()).setDuration(2000);
        wave.animate().scaleY(100f).setInterpolator(new AccelerateDecelerateInterpolator()).setDuration(2000);
    }

    public void stopMusic() {
        baseGameSound.pause();
    }
}
