package it.bluescreen.roboat;

import android.content.Context;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.graphics.Rect;
import android.util.Log;

/**
 * Created by romanofranz on 21/01/17.
 */


public class Roboat {

    public final static int ROBOAT_WIDTH = 240;
    public final static int ROBOAT_HEIGHT = 240;
    public final static int scaleCollider = 50;
    private Bitmap bitmap;
    private int x;
    private int y;
    private double orientation;
    private double goalOrientation;
    private int maxY;
    private int minY;
    private boolean isAccelerating;

    private final int MIN_SPEED = 1;
    private final int MAX_SPEED = 20;

    private Rect polygonArea;

    private float xGoal, yGoal;
    private int yScreen;
    private float codaX;
    private float codaY;
    private int speed;
    private boolean startMoving=false;

    public Roboat(Context context, int xScreen, int yScreen) {
        this.yScreen = yScreen;
        x = xScreen/2;
        y = 250;

        xGoal = x;
        yGoal = y;
        orientation = 90;
        goalOrientation = 90;

        Bitmap bitmap1 = BitmapFactory.decodeResource(context.getResources(), R.drawable.boathex);
        bitmap = Bitmap.createScaledBitmap(bitmap1, ROBOAT_WIDTH, ROBOAT_HEIGHT, false);
        maxY = yScreen - bitmap.getHeight();
        minY = 0;

        polygonArea =  new Rect(x, bitmap.getHeight(), bitmap.getWidth(), y);
    }

    public void update() {
        speed = 5;

        if(!startMoving){
            speed = 0;
        }else {
            if (isAccelerating) {
                speed += 3;
            } else {
                speed -= 1;
            }
            if (speed > MAX_SPEED) {
                speed = MAX_SPEED;
            }

            if (speed < MIN_SPEED) {
                speed = MIN_SPEED;
            }
        }


        if(mustRotate()){

        }else {
            x += speed*Math.cos(Math.toRadians(orientation));
            y += speed*Math.sin(Math.toRadians(orientation));
        }

        polygonArea.left = x - bitmap.getWidth()/2;
        polygonArea.top = (int) (getYTranslated());
        polygonArea.right = x + bitmap.getWidth()/2;
        polygonArea.bottom = (int) (getYTranslated() + bitmap.getHeight());
        polygonArea.inset(scaleCollider, scaleCollider);
    }

    public Rect getPolygonArea() {
        return polygonArea;
    }

    public Bitmap getBitmap() { return bitmap; }

    public int getX() { return x; }
    public int getY() {
        return y;
    }

    public void moveToXY(float newX, float newY) {
        xGoal = newX;
        yGoal = yScreen - newY;
        startMoving = true;
        findOrientation();
    }

    private void findOrientation() {
        codaX = (float) (x - ROBOAT_HEIGHT*Math.cos(Math.toRadians(orientation)));
        codaY = (float) (y - ROBOAT_HEIGHT*Math.sin(Math.toRadians(orientation)));

        //Log.d("ANGLE", x + "\n" + y);
        //Log.d("ANGLE", codaX + "\n" + codaY);

        float newX = xGoal - codaX;
        float newY = yGoal - codaY;
        float puntaX = x - codaX;
        float puntaY = y - codaY;
        orientation = Math.toDegrees(Math.atan2(puntaY, puntaX));
        goalOrientation = Math.toDegrees(Math.atan2(newY, newX));
        //TODO modificare per rotazione

        orientation = goalOrientation;
        //Log.d("ANGLE", orientation + "\n" + goalOrientation + "\n" + getAngleToRotate() + "\n" + bitmap.getWidth() + "\n" + bitmap.getHeight());
    }

    public boolean mustRotate(){
        return Math.abs(getAngleToRotate()) > 2;
    }

    public double getAngleToRotate(){
        return orientation - goalOrientation;
    }

    public float getCenterX(){
        return x;
    }

    public float getCenterY(){
        return getYTranslated() + ROBOAT_HEIGHT/2;
    }

    public float getYTranslated(){
        return yScreen - y;
    }

    public void changeOrientation(float deg){
        orientation += deg;
        Log.d("ANGLE", orientation + "\n" + goalOrientation + "\n" + getAngleToRotate() + "\n" + bitmap.getWidth() + "\n" + bitmap.getHeight());
        x += Math.cos(Math.toRadians(Math.abs(orientation)));
        y += Math.sin(Math.toRadians(Math.abs(orientation)));
    }

    public float getCodaX() {
        return codaX;
    }

    public float getCodaY() {
        return codaY;
    }
    public float getCodaYTranslated() {
        return yScreen - codaY;
    }

    public double getOrientation() {
        return orientation;
    }

    public void setAccelerating(boolean accelerating) {
        isAccelerating = accelerating;
    }
}
